/*  File Cleaner Code File.
	An application to clean files in a specified folder or in the Temporary
    Internet Files using deleting, wiping, and DES wiping.
    (c) 2004, Alfred P. Reaud
    Created: Thursday, November 11, 2004
    Current: Tuesday, December 7, 2004
    Version: 1.0.1.3

	This program is free software; you can redistribute it and/or modify
  	it under the terms of the GNU General Public License as published by
  	the Free Software Foundation; either version 2 of the License, or
  	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
  	but WITHOUT ANY WARRANTY; without even the implied warranty of
  	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
  	along with this program; if not, 
    browse to http://www.gnu.org/copyleft/gpl.html
    or write to the:
         Free Software Foundation, Inc.,
         59 Temple Place - Suite 330,
         Boston, MA 02111-1307, USA
*/
#include "FileCleaner.h"

using namespace std;

CODE code;

CODE::CODE ( void ) {
/* This is the class constructor for the CODE class.  It initializes variables as
   necessary for this class.
   Alfred P. Reaud, 05/12/04 17:25
*/
 int i = 0;

 	prompt = "";
 	// Null out the number conversion buffers.
 	for ( i = 0; i != NUMBER_BUFFER_SIZE; i++ ) {
 	    numberbufferfloat[i] = NULL_CHAR;
      	numberbufferint[i] = NULL_CHAR;
       	numberbufferlong[i] = NULL_CHAR;
 	}
    numberptrf = numberbufferfloat;
  	numberptri = numberbufferint;
  	numberptrl = numberbufferlong;
}


void CODE::process_common( long filecount, string filenames ) {
/* This procedure generalizes the processing of deleting, wiping, and DES wiping.
   - filecount is the count of files to process.
   - filenames is the string containing the names of files to operate on seperated
     by newline characters.
   Alfred P. Reaud, 09/12/04 13:38
*/
 float minutes = 0;
 float seconds = 0;
 float estimatedtime = 0;
 long wipecount = 0;
 long deletecount = 0;

    // Display an estimeted time.
    if (semaphore.enableDESflag) estimatedtime = ((float)filecount) * TIMEPERDESWIPE;
    if (semaphore.enablewipeflag) estimatedtime = ((float)filecount) * TIMEPERWIPE;
    if (semaphore.enabledeleteflag) estimatedtime += ((float)filecount) * TIMEPERDELETE;
    if (estimatedtime >= ESTTIMELIM) {
        // Display only if greater that 100ms.
        prompt = (string)"\nEstimated time: ";
     	if ( estimatedtime > SHOWMIN ) {
     	    // Convert elapsed time to minutes:seconds display.
     	    seconds = modf( estimatedtime/SECONDSINMIN, &minutes);
     	    prompt += const_cast<char *>(itoa((int)minutes, numberptri, BASE_TEN)) + (string)" min. ";
     	    prompt += gcvt(seconds * SECONDSINMIN, 5, numberptrf) + (string)" sec.";
     	}
     	else prompt += gcvt(estimatedtime, 6, numberptrf) + (string)" seconds.";
     	gui.write_textDisplay(prompt);
    }    
    // Set the process bar divider to the count of file operations.
    if (semaphore.enableDESflag) gui.set_prog_divider( (float)(filecount * 6) );
    else if (semaphore.enablewipeflag) gui.set_prog_divider( (float)(filecount * 2) );
    else if (semaphore.enabledeleteflag) gui.set_prog_divider( (float)filecount );
    // Seperate these so that we assure flushing to the disk.
    if (semaphore.enableDESflag) {
       prompt = (string)"\nDES Wiping " + ltoa(filecount, numberptrl, BASE_TEN);
       (filecount == 1) ?  prompt += (string)" file." : prompt += (string)" files.";
       gui.write_textDisplay(prompt);
       // Do DES file wiping.
       wipecount = util.wipe_files(true, filecount, filenames);
       prompt = (string)"\n" + ltoa(wipecount, numberptrl, BASE_TEN);
       (wipecount == 1) ?  prompt += (string)" file was" : prompt += (string)" files where";
       prompt += (string)" DES wiped.";
       gui.write_textDisplay(prompt);
    }
    if (semaphore.enablewipeflag) {
       prompt = (string)"\nWiping " + ltoa(filecount, numberptrl, BASE_TEN);
       (filecount == 1) ?  prompt += (string)" file." : prompt += (string)" files.";
       gui.write_textDisplay(prompt);
       // Do normal file wiping.
       wipecount = util.wipe_files(false, filecount, filenames);
       prompt = (string)"\n" + ltoa(wipecount, numberptrl, BASE_TEN);
       (wipecount == 1) ?  prompt += (string)" file was" : prompt += (string)" files where";
       prompt += (string)" wiped.";
       gui.write_textDisplay(prompt);
    }
  	if (semaphore.enabledeleteflag) {
       prompt = (string)"\nDeleting " + ltoa(filecount, numberptrl, BASE_TEN);
       (filecount == 1) ?  prompt += (string)" file." : prompt += (string)" files.";
       gui.write_textDisplay(prompt);
       // Do file deleting.
       deletecount = util.delete_files(filecount, filenames);
       prompt = (string)"\n" + ltoa(deletecount, numberptrl, BASE_TEN);
       (deletecount == 1) ?  prompt += (string)" file was" : prompt += (string)" files where";
       prompt += (string)" deleted.";
       gui.write_textDisplay(prompt);
    }
    // Display the error statistics.
    prompt = (string)"\nThere ";
    (parameters.deniedcount == 1) ? prompt += (string)"was " : prompt += (string)"where ";
    prompt += ltoa(parameters.deniedcount, numberptrl, BASE_TEN);
    prompt += (string)" read only ";
    (parameters.deniedcount == 1) ? prompt += (string)"file" : prompt += (string)"files";
    prompt += ".\nThere ";
    (parameters.accesserrorcount == 1) ? prompt += (string)"was " : prompt += (string)"where ";
    prompt += ltoa(parameters.accesserrorcount, numberptrl, BASE_TEN);
    prompt += (string)" access ";
    (parameters.accesserrorcount == 1) ? prompt += (string)"error" : prompt += (string)"errors";
    prompt += (string)".\nThere ";
    (parameters.othererrcount == 1) ? prompt += (string)"was " : prompt += (string)"where ";
    prompt += ltoa(parameters.othererrcount, numberptrl, BASE_TEN);
    prompt += (string)" ownership ";
    (parameters.othererrcount == 1) ? prompt += (string)"error.\n" : prompt += (string)"errors.\n";
    prompt += ltoa(parameters.fileinusecount, numberptrl, BASE_TEN);
    (parameters.fileinusecount == 1) ?  prompt += (string)" file" : prompt += (string)" files";
    prompt += (string)" may be locked by other applications.";
	gui.write_textDisplay(prompt);
}

 
void CODE::process_named ( void ) {
/* User has indicated a named folder.  Use it if possibe.  Serious error checking here,
   as we can trash the wrong directory easily!  Assume exceptions on all calls due 
   to removable media such as flash drives.
   Alfred P. Reaud, 05/12/04 17:34
*/
 char *entrydir;
 char *testdir;

 char *path = parameters.useFolder;

 int errornumber = 0;
 int i;
 int pathlength = strlen(path);

 long filecount = 0;

 string reason;
 string usefolderfilenames = "";

	// First check for too long of a path string.
	if(pathlength > MAX_FOLDER_TEXT_SIZE) {
	    prompt = (string)"The given path " + quote + path + quote + (string)" is too long, ";
	    prompt += itoa(pathlength, numberptri, BASE_TEN);
 	    prompt += (string)" characters.\nIt must be less than or equal to MAX_FOLDER_TEXT_SIZE characters.";
		fl_alert(prompt.c_str());
  		return;
	}

    // Check for prohibited folders.
    if ( stricmp(path, Prohibited[0].c_str()) == 0 ) {
        prompt = (string)"The path " + quote + path + quote + prohibtext;
		fl_alert(prompt.c_str());
  		return;
    }
    for (i = 1; i != PROHIB_COUNT; i++) {
    	if ( strstr(strlwr(path), Prohibited[i].c_str()) != NULL ) {
        prompt = (string)"The path " + quote + path + quote + prohibtext;
    		fl_alert(prompt.c_str());
      		return;
    	}
    }
 	// Save the current path.
 	entrydir = getcwd(NULL, MAX_FOLDER_TEXT_SIZE);
    // Check for default folder usage.
	if ( ((char)path[pathlength-1] == ':') && (pathlength == 2) ) {
 		chdir(path);
   		testdir = getcwd(NULL, MAX_FOLDER_TEXT_SIZE);
 		// Again check for prohibited folders.
	    if ( ((char)path[0] == 'C' ) || ((char)path[0] == 'c') ) {
            if ( stricmp(testdir, Prohibited[0].c_str()) == 0 ) {
                prompt = prohibprepath + quote + path + quote + prohibpostpath;
                prompt += quote + testdir + quote + prohibtextdef;
        		fl_alert(prompt.c_str());
          		chdir(entrydir);
            	free(entrydir);
             	free(testdir);
          		return;	
            }
            for (i = 1; i != PROHIB_COUNT; i++) {
            	if ( strstr(strlwr(testdir), Prohibited[i].c_str()) != NULL ) {
                	prompt = prohibprepath + quote + path + quote + prohibpostpath;
                    prompt += quote + testdir + quote + prohibtextdef;
            		fl_alert(prompt.c_str());
              		chdir(entrydir);
                	free(entrydir);
                 	free(testdir);
              		return;
            	}
           	}
       	}
       	// Update entries to indicate actual folder.
       	parameters.useFolder = testdir;
       	gui.usefolderTextBox->value(testdir);
        free(testdir);
        // Verify with user. 
	    prompt = (string)"Please confirm that you wish to operate on the default folder on the ";
	    prompt += quote + path + quote + (string)" drive.";
	    if(!fl_ask(prompt.c_str())) return;
	}
 	
    // Now try to open the folder. 	
	if(chdir(path)) {
		errornumber = errno;
		// Folder not present, snatch the error number.
		prompt = (string)"\nThe folder you specified, " + quote + path + quote;
  		prompt += (string)" was not found!  The reason was:\n";
    	switch(errornumber) {
     		case EROFS:
      		case EEXIST:
  		    case ENOTDIR:
        	case EISDIR:
        	case ENOTEMPTY:
      			reason = (string)"The directory specified by is read-only or is not empty.";
      			break;
   			case EPERM:
   			case EACCES:
          		reason = (string)"You do not have the required permission, the media is write protected or not present.";   
          		break;
    		case EBADF:
		    case EINVAL:
    		    reason = quote + path + quote;
    		    reason += (string)" is a zero-length string, contains only white space, or contains one or more invalid characters.";
    		    break;
		    case EFAULT:
		        reason = quote + path + quote +(string)" is a null reference.";
          		break;
    		case ENAMETOOLONG:
    		    reason = (string)"The specified path exceeds the system-defined maximum length.";
          		reason += (string)"\nOn Windows-based platforms, paths must be less than MAX_FOLDER_TEXT_SIZE characters.";
           		break;
    		case ENXIO:
		    case EAGAIN:
      		case ENODEV:
      		case ENOFILE:
        		reason = (string)"The specified path is invalid, or it's on an unmapped drive.";
          		break;
    		case ENOSYS:
    		    reason = (string)"Creating a directory with only the colon(:) character was attempted.";
          		break;
    		default:
    		    reason = (string)"Exception not handled by this application.";
	    }
			// An exception occurred, so bug out after notifying the user!
			prompt += reason;
			prompt += (string)"\nTherefore I can't continue!\nError Number = ";
   			prompt += itoa(errornumber, numberptri, BASE_TEN) + (string)", " + strerror(errornumber);
   			prompt += (string)".";
			fl_alert(prompt.c_str());
			free(entrydir);
			return;
	}

 	// Update status window.
 	prompt = (string)"\nProcessing " + quote + path + quote;
 	gui.write_textDisplay(prompt);

	/* Folder is present, so first we have to scan for files.  Once we do this, then
	   we will display the scan.  After that we either perform a DES wipe or a normal 
       wipe as indicated.  Then we delete the files if required, and finally we display
       the results.
    */
    if( (filecount = util.search_for_files(path, (string)"*.*", &usefolderfilenames)) == 0 ) {
 	    // Update status window.
 	    prompt = (string)"\nNo files found.";
 	    gui.write_textDisplay(prompt);
        chdir(entrydir);  // Restore the entry folder.	
        free(entrydir);
        return;
    }
    // Update the status.
    prompt = (string)"\nFound " + ltoa(filecount, numberptrl, BASE_TEN);
    (filecount == 1) ?  prompt += (string)" file!" : prompt += (string)" files!";
    gui.write_textDisplay(prompt);
    // Do it!
    process_common(filecount, usefolderfilenames);
	// Restore the entry folder.
  	chdir(entrydir);
  	free(entrydir);
}


void CODE::process_ok ( void ) {
/* Processing for Start button command.  Here we clear all pertinent flags and
   initialize any other value that may give us problems.  On exit we update the
   text display to show elapsed time and indicate an idle application.
   Alfred P. Reaud, 09/12/04 13:36
*/

 float minutes = 0;
 float seconds = 0.0;
 clock_t startt, endt = 0;
 float elapsedtimesec = 0.0;

 	// Get elapsed time.
  	startt = clock();

  	// Clear counts, flags, and strings.
	semaphore.abortflag = false;
	semaphore.inloopflag = false;
	parameters.accesserrorcount = 0;
	parameters.deniedcount = 0;
	parameters.othererrcount = 0;
	parameters.fileinusecount = 0;
	gui.clear_progress();
	semaphore.statechangedflag = true;

 	// Process either TIF or named folder.
 	if(semaphore.doTIFflag) process_TIF();
	else process_named();

	// Show elapsed time and end.
 	endt = clock();
 	elapsedtimesec = ((float)(endt - startt))/((float)CLOCKS_PER_SEC);
 	prompt = "\nElapsed time: ";
 	if ( elapsedtimesec > SHOWMIN ) {
 	    // Convert elapsed time to minutes:seconds display.
 	    seconds = modf( elapsedtimesec/SECONDSINMIN, &minutes);
 	    prompt += const_cast<char *>(itoa((int)minutes, numberptri, BASE_TEN)) + (string)" minutes ";
 	    prompt += gcvt(seconds * SECONDSINMIN, 5, numberptrf) + (string)" seconds.";
 	}
  	else prompt += gcvt(elapsedtimesec, 6, numberptrf) + (string)" seconds.";
 	prompt += gui.appIdleprompt;
	gui.write_textDisplay(prompt);
}


void CODE::process_TIF ( void ) {
/* Process in Windows Temporary Internet file area.
   Alfred P. Reaud, 05/12/04 17:45
*/
 char *entrydir = NULL;
 
 int i;

 long foldercount = 0;
 long filecount = 0;
 unsigned long lastpos = 0;
 unsigned long nlpos = 0;

 string currentfolder = "";
 string filenames = "";
 string foldernames = "";

 	// Update status window.
 	prompt = (string)"\nSearching for temporary internet folders...";
 	gui.write_textDisplay(prompt);

 	// Save the current path.
 	entrydir = getcwd(NULL, MAX_FOLDER_TEXT_SIZE);
 	// Scan the folder tree for pertinent folders.
 	for (i = 0; i != SEARCH_COUNT; i++) {
  		foldercount += util.recurse_search_for_folders(TIFsearchStart, TIFsearch[i], &foldernames);
 	}
 	prompt = (string)"\nFound ";
    prompt += ltoa(foldercount, numberptrl, BASE_TEN);
  	(foldercount == 1) ?  prompt += (string)" folder" : prompt += (string)" folders";
  	prompt += (string)" that may contain temporary interner files!";
    prompt += (string)"\nSearching for files...";
    gui.write_textDisplay(prompt);

    // Now scan the folders for files.
  	for (i = 0; i != foldercount; i++) {
  	    if( (nlpos = foldernames.find("\n",lastpos)) == string::npos ) break;
  	    currentfolder = foldernames.substr(lastpos, nlpos-lastpos);
  	    lastpos = nlpos + 1;
       	filecount += util.search_for_files(currentfolder, (string)"*.*", &filenames);
   	}
    if(filecount == 0) {
 	    // Update status window.
 	    prompt = (string)"\nNo files found.";
 	    gui.write_textDisplay(prompt);
        return;
    }
    // Update the status.
 	prompt = (string)"\nFound ";
    prompt += ltoa(filecount, numberptrl, BASE_TEN);
    (filecount == 1) ?  prompt += (string)" file!" : prompt += (string)" files!";
    gui.write_textDisplay(prompt);

    // Do it!
    process_common(filecount, filenames);
    // Restore the entry folder.
  	chdir(entrydir);
  	free(entrydir);
}
