/*  File Cleaner Main File.
	An application to clean files in a specified folder or in the Temporary
    Internet Files using deleting, wiping, and DES wiping.
    (c) 2004, Alfred P. Reaud
    Created: Thursday, November 11, 2004
    Current: Tuesday, December 7, 2004
    Version: 1.0.1.3


	This program is free software; you can redistribute it and/or modify
  	it under the terms of the GNU General Public License as published by
  	the Free Software Foundation; either version 2 of the License, or
  	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
  	but WITHOUT ANY WARRANTY; without even the implied warranty of
  	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
  	along with this program; if not,
    browse to http://www.gnu.org/copyleft/gpl.html
    or write to the:
         Free Software Foundation, Inc.,
         59 Temple Place - Suite 330,
         Boston, MA 02111-1307, USA
*/
#include "FileCleaner.h"

using namespace std;

// Global structures that become invisible if not delcared as extern in the
// header file.
Flag semaphore = {false};
Value parameters = {0};

GUI gui;

GUI::GUI ( void ) {
/* This is the class constructor for the GUI class.  It initializes variables as 
   necessary for this class.
   Alfred P. Reaud, 05/12/04 19:09
*/
 int i = 0;
 	
 	// Null out the useFolderbuffer.
 	for (i = 0; i != TEXT_BUFFER_SIZE; i++) useFolderbuffer[i] = NULL_CHAR;
 	parameters.useFolder = useFolderbuffer;
 	// Clear the prompt string.
	prompt = "";
	// Initialize class strings.
	appIdleprompt = "\nApplication idle.\n";
	init_appIdleprompt = "Application idle.";
	// Initialize the progress variables.
	progress.count = 0.0;
	progress.divider = 1.0;
	// Initialize the widget pointers.
    mainWindow = (Fl_Double_Window *)0;
    doTIFRadioButton = (Fl_Round_Button *)0;
    usefolderTextBox = (Fl_Input *)0;
    enabledeleteCheckBox = (Fl_Check_Button *)0;
    enablewipeCheckBox = (Fl_Check_Button *)0;
    enableDESCheckBox = (Fl_Check_Button *)0;
    applicationProgressBar = (Fl_Progress *)0;
    startCommandButton = (Fl_Light_Button *)0;
    abortCommandButton = (Fl_Button *)0;
    cancelCommandButton = (Fl_Button *)0;
    commandbuttonGroup = (Fl_Group *)0;
    statusTextDisplay = (Fl_Text_Display *)0;
    // Now construct the window!
    make_window();
}


/* -----------------------------------------------------------------------------
   All procedures in this module declared as static void are callbacks.  They are
   declared static void to allow for calling by the handlers for the different
   widgets and system.
*/
static void cb_applicationChecker ( void* ) {
/* This callback updates the application window.
   Alfred P. Reaud, 05/12/04 16:28
*/
    if(!semaphore.statechangedflag) return;
    semaphore.statechangedflag = false;
    if(semaphore.inloopflag) gui.applicationProgressBar->redraw();
    else gui.mainWindow->redraw();
}


static void cb_mainWindow ( Fl_Window* topWindow, void* ) {
/* Main window callback in FLTK.  Pretty much all we do here is terminate the
   application.
   Alfred P. Reaud, 28/11/04 12:43
*/
	topWindow->hide();
	exit(0);
}


static void cb_doTIFRadioButton ( Fl_Round_Button* TIFRadioButton, void* ) {
/* Do TIF Radio Button callback.  Toggles value of button and modifies
   global variable.  Disables useFolderTextBox if true.
   Alfred P. Reaud, 05/12/04 16:30
*/

	if ( TIFRadioButton->value() ) {
 		semaphore.doTIFflag = true;
    	gui.usefolderTextBox->deactivate();
  	}
	else {
 		semaphore.doTIFflag = false;
    	gui.usefolderTextBox->activate();
   	}
	// Update the display.
    semaphore.statechangedflag = true;
    Fl::check();
}


static void cb_usefolderTextBox ( Fl_Input* usefolder, void* ) {
/* Use folder testbox.  Allows entry of a folder name to use.  Disabled if
   doTIFRadioButton is true.
   Alfred P. Reaud, 30/11/04 20:18
*/
	parameters.useFolder = strcpy( parameters.useFolder, usefolder->value() );
}


static void cb_enabledeleteCheckBox ( Fl_Check_Button* enabledelete, void* ) {
/* Check box to enable file deletes.  If this box is not checked, the application counts
   the number of files present in the Temporary Internet Folders or in the specified
   folder.  This check box will be automatically set if you check either the enable
   wipe or enable DES wipe checkboxes.  Clearing this checkbox clears the other enable
   check boxes.
   Alfred P. Reaud, 05/12/04 16:32
*/
 	if (enabledelete->value()) semaphore.enabledeleteflag = true;
	else {
     	semaphore.enabledeleteflag = false;
		semaphore.enablewipeflag = false;
		gui.enablewipeCheckBox->clear();
		semaphore.enableDESflag = false;
		gui.enableDESCheckBox->clear();
	}
	// Update the display.
	semaphore.statechangedflag = true;
	Fl::check();
}


static void cb_enablewipeCheckBox ( Fl_Check_Button* enablewipe, void* ) {
/* Check box to enable file wipes.  If this box is checked, wiping will occur on the
   files present in the Temporary Internet Folders or in the specified folder.  This
   check box will automatically set the enable wipe checkbox, and clear the enable
   DES wipe checkbox.  This checkbox will be cleared if the enable delete checkbox
   is cleared.
   Alfred P. Reaud, 05/12/04 16:33
*/
	if (enablewipe->value()) {
		semaphore.enablewipeflag = true;
       	semaphore.enabledeleteflag = true;
       	gui.enabledeleteCheckBox->set();
       	if (semaphore.enableDESflag) {
           	semaphore.enableDESflag = false;
           	gui.enableDESCheckBox->clear();
       	}
 	}
	else semaphore.enablewipeflag = false;
	// Update the display.
	semaphore.statechangedflag = true;
	Fl::check();
}


static void cb_enableDESCheckBox ( Fl_Check_Button* enableDES, void* ) {
/* Check box to enable wipes using the DES specification.  If this box is checked,
   wiping will occur on the files present in the Temporary Internet Folders or 
   in the specified older.  This check box will automatically set the enable delete
   checkbox, and clear the enable wipe checkbox.  This checkbox will be cleared if
   the enable delete checkbox is cleared.
   Alfred P. Reaud, 05/12/04 16:54
*/
	if (enableDES->value()) {
		semaphore.enableDESflag = true;
      	semaphore.enabledeleteflag = true;
       	gui.enabledeleteCheckBox->set();   
 	    if (semaphore.enablewipeflag) {
 	        semaphore.enablewipeflag = false;
 	        gui.enablewipeCheckBox->clear();
 	    }
 	}
	else semaphore.enableDESflag = false;
	// Update the display.
	semaphore.statechangedflag = true;
	Fl::check();
}


static void cb_startCommandButton ( Fl_Light_Button* startCommand, void* ) {
/* Start processing command.  Further, all the user interface is disabled except
   for the Abort command while processing is occuring.  The user is prompted if they
   have not checked the doTIF radio rutton and no entry is in the useFolder text box
   Alfred P. Reaud, 05/12/04 16:56
*/
 char test = (char)*parameters.useFolder;

	// Though this repeates prior code, it allows for reuse of the start command.
	if ( gui.doTIFRadioButton->value() ) semaphore.doTIFflag = true;
	else semaphore.doTIFflag = false;
 	if ( gui.enabledeleteCheckBox->value() ) semaphore.enabledeleteflag = true;
	else semaphore.enabledeleteflag = false;
	if ( gui.enablewipeCheckBox->value() ) semaphore.enablewipeflag = true;
	else semaphore.enablewipeflag = false;
	if ( gui.enableDESCheckBox->value() ) semaphore.enableDESflag = true;
	else semaphore.enableDESflag = false;
	if( (!semaphore.doTIFflag) && (test == NULL_CHAR) ) {
    	fl_alert("If you did not select find temporary internet files then you must enter a folder name to use!");   	
     	return;
	}

	// Configure the widgets then go for it!
 	gui.disableWidgets();
	code.process_ok();
	gui.enableWidgets();
}


static void cb_abortCommandButton(Fl_Button* abort, void*) {
/* Aborts a command in process and return to GUI.
   Alfred P. Reaud, 05/12/04 16:56
*/  
	semaphore.abortflag = true;
 	gui.enableWidgets();
}


static void cb_cancelCommandButton(Fl_Button* cancel, void*) {
/* Cancel Command.  Should close window and terminate the application.
   Alfred P. Reaud, 05/12/04 16:56
*/
	gui.mainWindow->hide();
	exit(0);
}


// -----------------------------------------------------------------------------
void GUI::make_window(void) {
/* Creates the main window and widgets.
   Alfred P. Reaud, 05/12/04 16:53
*/

	/* TODO (20041120#7#): Add check button and logic for "FULL Wipe", with 
	                       red text tooltip warning to read documantation. */
	mainWindow = new Fl_Double_Window(382, 338, "File Cleaner 1.0.1.3");
	mainWindow->callback((Fl_Callback*)cb_mainWindow);
	mainWindow->box(FL_PLASTIC_UP_BOX);
	mainWindow->align(FL_ALIGN_LEFT);
	
	mainWindow->begin();
	doTIFRadioButton = new Fl_Round_Button(10, 15, 25, 25, "Operate on Temporary Internet Files");
   	doTIFRadioButton->callback((Fl_Callback*)cb_doTIFRadioButton);
	doTIFRadioButton->when(FL_WHEN_RELEASE);
  	doTIFRadioButton->down_box(FL_ROUND_DOWN_BOX);
 	doTIFRadioButton->tooltip("Click here to count or wipe the Temporary Internet Folders.");
  	doTIFRadioButton->clear();

    usefolderTextBox = new Fl_Input(15, 60, 355, 25, "Operate in Folder:");
    usefolderTextBox->callback((Fl_Callback*)cb_usefolderTextBox);
    usefolderTextBox->when(FL_WHEN_CHANGED);
    usefolderTextBox->align(FL_ALIGN_TOP_LEFT);
    usefolderTextBox->tooltip("Input name of folder to count or wipe.");
	usefolderTextBox->value("\0");

    enabledeleteCheckBox = new Fl_Check_Button(15, 95, 25, 20, "Enable Delete");
    enabledeleteCheckBox->callback((Fl_Callback*)cb_enabledeleteCheckBox);
    enabledeleteCheckBox->when(FL_WHEN_RELEASE);
    enabledeleteCheckBox->down_box(FL_DOWN_BOX);
    enabledeleteCheckBox->align(FL_ALIGN_RIGHT);
    enabledeleteCheckBox->tooltip("Check to enable deleting, otherwise you are counting files.");
  	enabledeleteCheckBox->clear();

    enablewipeCheckBox = new Fl_Check_Button(145, 95, 25, 20, "Enable Wipe");
    enablewipeCheckBox->callback((Fl_Callback*)cb_enablewipeCheckBox);
    enablewipeCheckBox->when(FL_WHEN_RELEASE);
    enablewipeCheckBox->down_box(FL_DOWN_BOX);
    enablewipeCheckBox->align(FL_ALIGN_RIGHT);
    enablewipeCheckBox->tooltip("Enable file wiping.");
  	enablewipeCheckBox->clear();

    enableDESCheckBox = new Fl_Check_Button(145, 120, 25, 20, "Enable DES Wipe");
    enableDESCheckBox->callback((Fl_Callback*)cb_enableDESCheckBox);
    enableDESCheckBox->when(FL_WHEN_RELEASE);
    enableDESCheckBox->down_box(FL_DOWN_BOX);
    enableDESCheckBox->align(FL_ALIGN_RIGHT);
    enableDESCheckBox->tooltip("Check to enable DES wipe.");
  	enableDESCheckBox->clear();

    applicationProgressBar = new Fl_Progress(15, 150, 355, 20, "Progress:");
    applicationProgressBar->align(FL_ALIGN_TOP_LEFT);
    // Set to an initial value of 100%
    applicationProgressBar->maximum((float)1.0);
    applicationProgressBar->minimum((float)0.0);
    applicationProgressBar->value((float)0.0);

    commandbuttonGroup = new Fl_Group(5, 195, 88, 135);
    commandbuttonGroup->box(FL_ENGRAVED_FRAME);

    commandbuttonGroup->begin();

    startCommandButton = new Fl_Light_Button(15, 205, 65, 25, "Start");
    startCommandButton->callback((Fl_Callback*)cb_startCommandButton);
    startCommandButton->when(FL_WHEN_RELEASE);
    startCommandButton->align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
	startCommandButton->selection_color(FL_RED);
    startCommandButton->type(0);
    startCommandButton->tooltip("Start processing");

    abortCommandButton = new Fl_Button(15, 250, 65, 25, "ABORT");
    abortCommandButton->callback((Fl_Callback*)cb_abortCommandButton);
    abortCommandButton->when(FL_WHEN_RELEASE);
    abortCommandButton->align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
    abortCommandButton->tooltip("Abort processing in progress.");
    abortCommandButton->deactivate();

    cancelCommandButton = new Fl_Button(15, 295, 65, 25, "Cancel");
    cancelCommandButton->callback((Fl_Callback*)cb_cancelCommandButton);
    cancelCommandButton->when(FL_WHEN_RELEASE);
    cancelCommandButton->align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
    cancelCommandButton->tooltip("Cancel application.");

    commandbuttonGroup->end();

    statusTextDisplay = new Fl_Text_Display(100, 195, 270, 135, "File Cleaner Status/Results");
    statusTextDisplay->box(FL_ENGRAVED_FRAME);
    statusTextDisplay->buffer( &textbuffer );
    statusTextDisplay->textcolor(FL_BLUE);
    statusTextDisplay->textfont(FL_TIMES);

    mainWindow->end();

    Fl::visual(FL_DOUBLE|FL_INDEX);
    mainWindow->show();
}


void GUI::disableWidgets(void) {
/* Disables all pertinent widgets at once.
   Alfred P. Reaud, 05/12/04 16:47
*/
 	doTIFRadioButton->deactivate();
  	usefolderTextBox->deactivate();
  	enabledeleteCheckBox->deactivate();
   	enablewipeCheckBox->deactivate();
    enableDESCheckBox->deactivate();
    startCommandButton->deactivate();
	cancelCommandButton->deactivate();
	// Activeate the abort button as now it's pertinent.
	abortCommandButton->activate();
	// Update the display.
	semaphore.statechangedflag = true;
	Fl::check();
}


void GUI::enableWidgets(void) {
/* Enables all pertinent widgets at once.
   Alfred P. Reaud, 05/12/04 16:45
*/
 	doTIFRadioButton->activate();
 	// Only enable the useFolderTextBox if the doTIF radio button is off.
  	if(!semaphore.doTIFflag) usefolderTextBox->activate();
   	enabledeleteCheckBox->activate();
   	enablewipeCheckBox->activate();
    enableDESCheckBox->activate();
	startCommandButton->activate();
	cancelCommandButton->activate();
	// Deactiveate the abort button as we're not processing.
	abortCommandButton->deactivate();
	// Update the display.
	semaphore.statechangedflag = true;
	Fl::check();
}


void GUI::clear_progress( void ) {
/* This procedure clears the value of the progerss bar to 0% progress.
   Alfred P. Reaud, 04/12/04 19:14
*/
	progress.count = 0.0;
	applicationProgressBar->value(progress.count);
	semaphore.statechangedflag = true;
	Fl::check();
}


void GUI::set_prog_divider ( float value )  {
/* This procedure both sets the divider value for progress computation and due
   to that change clears the value of the progerss bar to 0% progress.
   - value is a float that indicates the numebr of files to be worked on.	
   Alfred P. Reaud, 04/12/04 19:19
*/
	progress.divider = value;
 	clear_progress();
}


void GUI::update_progress( void ){
/* This procedure generalizes the updating of the progress bar.  It uses the static
   variable progresscount.  This variable should be cleared by the processing start call.
   Alfred P. Reaud, 05/12/04 16:57
*/
float result;

 	progress.count += 1.0;
 	result = progress.count/progress.divider;
  	applicationProgressBar->value(result);
   	semaphore.statechangedflag = true;
   	Fl::check();
}


void GUI::write_textDisplay (string outtext) {
/* This procedure bring the text display writing method to one place.
   - outext is the text to write.
   Alfred P. Reaud, 05/12/04 17:00
*/
 	statusTextDisplay->insert(outtext.c_str());
 	statusTextDisplay->show_insert_position();
 	semaphore.statechangedflag = true;
 	Fl::check();
}


// -----------------------------------------------------------------------------
int main(void) {
/* Application main entry point.
   Alfred P. Reaud, 01/12/04 20:05
*/
    Fl::add_check(cb_applicationChecker);
    // Don't use the prompt constant appIdleprompt, as you don't want a newline
    // in the first line.
    gui.write_textDisplay(gui.init_appIdleprompt);
    return Fl::run();
}
