/*  File Cleaner Utility File.
	An application to clean files in a specified folder or in the Temporary
    Internet Files using deleting, wiping, and DES wiping.
    (c) 2004, Alfred P. Reaud
    Created: Thursday, November 11, 2004
    Current: Tuesday, December 7, 2004
    Version: 1.0.1.3

	This program is free software; you can redistribute it and/or modify
  	it under the terms of the GNU General Public License as published by
  	the Free Software Foundation; either version 2 of the License, or
  	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
  	but WITHOUT ANY WARRANTY; without even the implied warranty of
  	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
  	along with this program; if not, 
    browse to http://www.gnu.org/copyleft/gpl.html
    or write to the:
         Free Software Foundation, Inc.,
         59 Temple Place - Suite 330,
         Boston, MA 02111-1307, USA
*/
#include "FileCleaner.h"

using namespace std;

UTIL util;

UTIL::UTIL ( void ) {
/* This is the class constructor for the UTIL class.  It initializes variables as
   necessary for this class.
   Alfred P. Reaud, 05/12/04 15:59
*/
	prompt = "";
}

bool UTIL::bad_flush(int commitret, string file) {
/* This procedure generalizes the check for the result of flushing so that we don't
   have a plethora of repetitive code!  It will return true if the user requests 
   loop termination, false otherwise.
   - commitret is the value returned by the commit function.
   - file is the name of the file being operated on.
   Alfred P. Reaud, 28/11/04 20:19
*/
	if (commitret != 0) {
	    prompt = (string)"File " + quote + file + quote + (string)" not flushed!";
	    prompt += (string)"\nDo you wish to continue?";
	    if( fl_ask(prompt.c_str()) == 0 ) return true;
    }
    return false;
}


long UTIL::delete_files (long filecount, string filenames) {
/* This procedure deletes the number of files specified by count using names
   specified in filenames.  The return value is the number of files processed.
   This procedure also sets value semaphores for permission denied, access, 
   and other errors.
   - count is the number of files to delete.
   - filenames is a string that contains the names of the files to delete,
     seperated by newline characters.
   Alfred P. Reaud, 05/12/04 17:50
*/
 long deletedcount = 0;
 long i = 0;
 unsigned long nlpos = 0;
 unsigned long lastpos = 0;

 string currentfile = "";

   semaphore.inloopflag = true;
   for (i = 0; i != filecount; i++) {
       	if (semaphore.abortflag) {
            semaphore.inloopflag = false;
            break;
        }
  	    // Exit loop if we are out of new lines!
  	    if( (nlpos = filenames.find("\n",lastpos)) == string::npos ) break;
  	    currentfile = filenames.substr(lastpos, nlpos-lastpos);
  	    lastpos = nlpos + 1;
  	    if ( remove(currentfile.c_str()) )  {
  	        switch (errno) {
  	            case EACCES:
  	                // Check for read only errors. 
                   	if ( _access(currentfile.c_str(), 2) == EOF ) parameters.deniedcount++;
                   	else parameters.othererrcount++;
  	                break;
                case ENOENT:
                    // No entry error.
                    parameters.accesserrorcount++;
                    break;
                default:
                    // Any other kind of error.
                    parameters.othererrcount++;
           }
       }
       else deletedcount++;
       gui.update_progress();
 	}
  	semaphore.inloopflag = false;

  	return deletedcount;
}


int UTIL::force_flush (FILE *file) {
/* This procedure generalizes the forced flushing of a ofstream.  The stream
   must be open before calling this function.  If no errors occur, the procedure
   returns 0.  If the procedure has a problem on the fflush or _commit calls,
   it will return EOF.  Finally, if the procedure cannot open a FILE structure
   for the file, it will return 1.  A return of 1 indicates that another file
   is operating on it, and it should be left alone.
   - filestream is a pointer to the filestream to be operated on.
   - file is the name of the file.
   Alfred P. Reaud, 07/12/04 21:07
*/
 int commitresult = 0;
 int filenumber = 0;
 int flushresult = 0;

	// Get the file number.
 	filenumber = fileno(file);
	// Flush to system.
	flushresult = fflush(file);
	// Commit to disk, verified 11/15/2004 due to change in speed!
	commitresult = _commit(filenumber);
	// Close the file.
	fclose(file);
	return (flushresult | commitresult);
}


int UTIL::found_file_process (char *name, string start, string *result) {
/* This procedure is the processing for when a file is found that isn't a folder.
   The procedure returns -1 for the
   . and .. folders, 0 for a folder and 1 for a file.  Every file found adds the 
   full path to the string buffer.  File names are seperated by the newline character.
   - name is a pointer to the filename.
   - start is the target folder name.
   - result is a pointer to the file search result string.
   Alfred P. Reaud, 29/11/04 16:18
*/
 int temp;
 struct stat fileinfo = {0};

	// Bypass the . and .. folders, as they are the current folder and the parent.
	if( (strcmp(name,".") == 0) || (strcmp(name, "..") == 0) ) return (-1);
	temp = stat(name, &fileinfo);
	if( !_S_ISDIR(fileinfo.st_mode) ) {
		// Found a file so check to see if it is the one we  want.
		*result = start + (string)"\\" + name + (string)"\n";
		return 1;
	}
	return 0;
}


void UTIL::no_file_alert (string path) {
/* A small procedure to allow for just one delcaration of the alert
   for use in multiple locations. This only has meaning when not in temporary
   internet file mode
   - path is the name of the path that was searched.
   Alfred P. Reaud, 28/11/04 19:54
*/
	if (!semaphore.doTIFflag) {
 		prompt = (string)"I didn't find any content to the folder ";
 		prompt += quote + path + quote;
 		prompt += (string)"\nTherefore I can't continue!";
 		fl_alert(prompt.c_str());
	}
}


long UTIL::recurse_search_for_folders(string startpoint, string searchname, string *resultstring) {
/* Calls the private recursive search for folders function.  It's just used to
   simplify the call to the recursive search function.
   - startpoint is the start of the search.
   - searchname is the file to search for.
   - resultstring is a pointer to a string which holds the results.
   Alfred P. Reaud, 01/12/04 22:49
*/
	return recurse_search_for_folders(startpoint, searchname, (string)"", resultstring);
}


long UTIL::recurse_search_for_folders(string startpoint, string searchname,
										string lastname, string *resultstring) {
/* Performs recursive entry into folders looking for folders.  We start with a top level folder,
   and we search for a folder name.  If we find any folders that match the name, we add them
   to a C++ string which should have enough room for it as it's limit is about 2^30 characters.
   - startpoint is the start of the search.
   - searchname is the file to search for.
   - lastname is the name of the last folder recursed into.
   - resultstring is a pointer to a string which holds the results.
   Alfred P. Reaud, 28/11/04 19:54
*/
 bool foundfile = false;

 char *entrydir = NULL;
 int result = 0;

 long foundcount = 0;
 long findhandle = 0;

 string recursename = "";
 string tempstring = "";

 _finddata_t finddata = {0};
 struct stat fileinfo = {0};
 
  	entrydir = getcwd(NULL, MAX_FOLDER_TEXT_SIZE);
	chdir(startpoint.c_str());	// Do this again in case we are recursing.
	if((findhandle = _findfirst("*.*", &finddata)) == EOF) {
		prompt = (string)"I didn't find any content to the folder " + quote;
		if (semaphore.doTIFflag) prompt += TIFsearchStart;
		else prompt += parameters.useFolder;
 		prompt += quote + (string)"\nThis application is written for Windows XP, so I can't continue!";
		fl_alert(prompt.c_str());
 		free(entrydir);
  		return 0;
	}
  	// Find the next folder or file.
   	while( ((result = _findnext(findhandle, &finddata)) == 0) && !semaphore.abortflag ) {
  		foundfile = true;
  		// Bypass the . and .. folders.
  		if( (strcmp(finddata.name,".") == 0) || (strcmp(finddata.name, "..") == 0) ) continue;
  		result = stat(finddata.name, &fileinfo);
		if( _S_ISDIR(fileinfo.st_mode) ) {
			// Found a folder so check to see if it is the one we  want.
			if (lastname.compare(finddata.name) == 0) {
			   	// Found the same name again.
			   	parameters.samenamecount++;
			   	if ( parameters.samenamecount == RECURSE_MAX_COUNT ) {
           			semaphore.stoprecurseflag = true;
       				break;
   				}
			}
			else parameters.samenamecount = 0;
			tempstring = startpoint +(string)"\\" + finddata.name;
			recursename = tempstring;
			// If the searchname is a * then we are looking in a found folder.
			if( (searchname == (string)"*") || (stricmp(finddata.name, searchname.c_str()) == 0) ) {
				// It is, so add it's full path to the return string. Only do this
				// if the path isn't already present.
 				if (resultstring->find(tempstring,0) == string::npos) {
    				*resultstring += tempstring +(string)"\n";
			    	foundcount++;
			    	// Now recursively search into the whole folder itself.
			    	parameters.recursecount++;
			    	foundcount += recurse_search_for_folders(recursename, (string)"*", finddata.name, resultstring);
			    	parameters.recursecount--;
				}
			}
			else {
    			// Recurse!
		    	parameters.recursecount++;
		    	foundcount += recurse_search_for_folders(recursename, searchname, finddata.name, resultstring);
		    	parameters.recursecount--;
			}
			// Control recursion.
			if ( semaphore.stoprecurseflag ) {
  				if (parameters.recursecount != 0) {
 					_findclose(findhandle);
    				chdir(entrydir);
   					free(entrydir);
   					return foundcount;
			    }
	    		else semaphore.stoprecurseflag = false;
   			}
   			Fl::check();
   		}
   		if(!foundfile) {
	   		prompt = (string)"I didn't find any content to the folder " + quote + TIFsearchStart;
 	   		prompt += quote + (string)"\nThis application is written for Windows XP, so I can't continue!";
 		   	fl_alert(prompt.c_str());
  	   		// Exit cleanly.
			_findclose(findhandle);
  			chdir(entrydir);
   			free(entrydir);
    		return 0;
   		}
	}
	// Exit cleanly.
	_findclose(findhandle);
 	chdir(entrydir);
  	free(entrydir);
  	return foundcount;
}


long UTIL::search_for_files (string startpoint, string searchname, string *resultstring) {
/* This procedure logs the files in a given folder.  If we find 
   any files that match the search name, we add them to a C++ string 
   which should have enough room for it as it's limit is about 2^30 characters.
   - startpoint is the folder to start the search from.
   - searchname is the name to search for.
   - resultstring is a pointer to a string which hold the names of the the files found.
   Alfred P. Reaud, 28/11/04 20:12
*/
 int result = 0;
 
 long foundcount = 0;
 long findhandle = 0;

 string tempstring = "";

 _finddata_t finddata;

	// Change folders again as we are not necessarily in the proper one.
	chdir(startpoint.c_str());
	if( (findhandle = _findfirst(searchname.c_str(), &finddata)) == EOF ) {
	    no_file_alert(startpoint);
		fl_alert(prompt.c_str());
	    return 0;
	}
	// Do the following because we drop the first one in the while loop.
	if ((result = found_file_process(finddata.name, startpoint, &tempstring)) > 0) {
		foundcount++;
		*resultstring += tempstring;
  	}
    semaphore.inloopflag = true;
  	while(((result = _findnext(findhandle, &finddata)) == 0) && !semaphore.abortflag ) {
  		/* Find the next folder or file.  Don't count folders as we are only looking
  		   for files here.
	   */
  		if ((result = found_file_process(finddata.name, startpoint, &tempstring)) < 0) continue;
  		foundcount += result;
		if (result > 0) *resultstring += tempstring;
		Fl::check();
   	}
    semaphore.inloopflag = false;
	if(foundcount == 0) no_file_alert(startpoint);
	// Exit cleanly.
	_findclose(findhandle);
  	return foundcount;
}


long UTIL::wipe_files ( bool DES, long count, string filenames ) {
/* This procedure wipes the number of files specified by count using names
   specified in filenames.  The return value is the number of files processed.
   This procedure also sets value semaphores for in use and access errors. Ofstream
   class is not being used as it was giving inconsistent results.
   - DES is a flag indicating use DES wipe if true, normal wipe otherwise.
   - count is the number of files to delete.
   - filenames is a string that contains the names of the files to delete,
   seperated by newline characters.
   Alfred P. Reaud, 07/12/04 21:19
*/
 /* TODO (20041120#6#): Add full wipe as explained in Secure Deletion of Data 
                        from Magnetic and Solid-State Memory.htm after 
                        reading the document. Make it for pay!*/
 FILE *file;

 bool exitloop = false;
 bool exitinnerloop = false;

 int j, k = 0;

 long charcount = 0;
 long i = 0;
 long wipedcount = 0;
 unsigned long lastpos = 0;
 unsigned long nlpos = 0;

 string currentfile = "";

 struct stat fileinfo;

 	semaphore.inloopflag = true;
  	for (i = 0; i != count; i++) {
       	if (exitloop || semaphore.abortflag) break;
  	    // Exit loop if we are out of new lines!
  	    if( (nlpos = filenames.find("\n",lastpos)) == string::npos ) break;
  	    currentfile = filenames.substr(lastpos, nlpos-lastpos);
  	    lastpos = nlpos + 1;
  	    // First open as input to get the length.
  	    if ( stat(currentfile.c_str(), &fileinfo) == EOF ) {
  	        parameters.accesserrorcount++;
  	        continue;
  	    }
  	    charcount = fileinfo.st_size;
  	    // Now reopen it for output.
    	if ( (file = fopen(currentfile.c_str(), "rb+")) == NULL ) {
    	    //Try reopening as a read file.
    	    if ( (file = fopen(currentfile.c_str(), "rb")) != NULL ) {
    	        // However, this could be a read only file.
         		parameters.fileinusecount++;
         		// Close it to insure that we don't lock it accidentally.
         		fclose(file);
    	    }
         	else parameters.accesserrorcount++;
        	continue;
    	}    
		if (DES) {
		    // Write the hex characters FF.
		    if ( write_data(false, 0xff, charcount, file) ) {
		        // If we can't write the data, bypass other parts.
          		fclose(file);
          		// Correct the progress bar.
          		for (j = 0; j != 5; j++) gui.update_progress();
          		continue;
    		}
		    if (bad_flush( force_flush(file), currentfile )) {
          		// Correct the progress bar.
          		for (j = 0; j != 5; j++) gui.update_progress();
		        exitloop = true;
		        break;
		    }
       		gui.update_progress();
       		// Reopen file and write the hex characters 00.
        	if ( (file = fopen(currentfile.c_str(), "rb+")) == NULL ) {
        	    // This is counted as another type of error as we should
             	// be read-write and accessible to get here.
             	parameters.othererrcount++;
          		// Correct the progress bar.
          		for (j = 0; j != 4; j++) gui.update_progress();
            	continue;
        	}    
		    if ( write_data(false, NULL_CHAR, charcount, file) ) {
          		fclose(file);
          		// Correct the progress bar.
          		for (j = 0; j != 4; j++) gui.update_progress();
          		continue;
    		}
		    if ( bad_flush(force_flush(file), currentfile) ) {
          		// Correct the progress bar.
          		for (j = 0; j != 4; j++) gui.update_progress();
		        break;
		    }
       		gui.update_progress();
       		// Write with random charactes 3 times.
		    for (j = 0; ((j != 3) && !semaphore.abortflag); j++) {
		        // Reopen the file.
            	if ( (file = fopen(currentfile.c_str(), "rb+")) == NULL ) {
            	    // This is counted as another type of error as we should
                 	// be read-write and accessible to get here.
                 	parameters.accesserrorcount++;
        			exitinnerloop = true;
                 	// Only try it once.
                	break;
            	}    
		    	if ( write_data(true, NULL_CHAR, charcount, file) ) {
          			fclose(file);
        			exitinnerloop = true;
		    	    break;
		    	}
		    	if ( bad_flush(force_flush(file), currentfile) ) {
        			exitinnerloop = true;
        			exitloop = true;
		    	    break;
		    	}    
       			gui.update_progress();
	    	}
	    	if (exitinnerloop) {
	    	    exitinnerloop = false;
	    	    // Correct the progress bar.
	    	    for (k = 0; k != (3 - j); k++) gui.update_progress();
	    	}
      		if (exitloop) break;
  		}
    	else {	// Non-DES wipe, write random characters once.
    		if ( write_data(true, NULL_CHAR, charcount, file) ) {
    		    fclose(file);
          		gui.update_progress();
          		continue;
    		}
		    if ( bad_flush(force_flush(file), currentfile) ) {
          		gui.update_progress();
		        break;
		    }
   			gui.update_progress();
     	}
       	wipedcount++;
  	}
 	semaphore.inloopflag = false;
  	return wipedcount;
}


bool UTIL::write_data (bool dorandom, char ch, long charcount, FILE *currentfile) {
/* This procedure generalizes the writing of data to a file.  The file to be
   operated on must be open and in the proper mode before calling this procedure.
   Any errors occuring during write will cause the procedure to exit with the value
   of true, otherwise it exits with the value of false.  C++ calls where not used as
   they where giving inconsistent results.
   - userandom if true will result in the use of random characters.
   - ch is the character to use if userandom is false.
   - count is the count of bytes to write.
   - currentfile is a pointer to an open output FILE stream.
   Alfred P. Reaud, 07/12/04 22:06
*/
 char data = 0;

 long i = 0;

	if (dorandom) srand(clock());
 	for(i = 0; ((i != charcount) && !semaphore.abortflag); i++) {
 		// Write the characters.
   		if (dorandom) data = rand();
   		else data = ch;
   		if ( fputc((unsigned)data, currentfile) == EOF ) return true;
    }
  	return false;
}
